==> ktoolkits-0.0.3.tar.gz: ktoolkits-0.0.3/ktoolkits/agent/function_call.py <==
import json
from ktoolkits.client.base_api import BaseApi
from ktoolkits.client.base_api import GetMixin
from typing import Union,Dict,List

class FunctionCall(BaseApi):

    @classmethod
    def create_tool(cls,tool_name: str,api_key: str = None) -> Union[Dict,None]:

        api_reponse = GetMixin.get(path="tool", api_key=api_key, params={"tool_name":tool_name})

        if api_reponse.status_code != 200:
            return None
        else:
            result = {}
            result["tool_name"] = api_reponse.output.get("tool_name")
            result["tool_args"] = cls._get_args_name(api_reponse.output.get("input_parameters"))
            result["tool_func"] = cls._create_function(tool_name,result["tool_args"])
            result["tool_desc"] = api_reponse.output.get("description")
            return result
    
    @classmethod
    def _get_args_name(cls, input_parameters: str)-> List[str]:
        argsJson = json.loads(input_parameters)
        args_list =[]
        for item in argsJson:
            args_name = item.get("name")
            args_list.append(args_name)
        return args_list
    @classmethod
    def _create_function(cls, tool_name: str, tool_args: List[str]):

        func_name = f"k_function_{tool_name}"

        args_names = tool_args

        input_args = {}
        for item in args_names:
            input_args[item] = None

        args_types = [Union[str,Dict[str,str]]]

        body = f"""
        import ktool
        a = ktool.Runner.call(
                tool_name='{tool_name}',
                tool_input={input_args},
                tool_type="docker",
        )
        tool_output = a.output
        if tool_output:
            return tool_output
        else:
            return a.message
        """
        return_type = "str"

        return cls._create_dynamic_function(func_name,args_names,args_types,body,return_type)

    @classmethod
    def _create_dynamic_function(cls,func_name,args_names,arg_types,body, return_type):
        def get_type_annotation(type_):
            if hasattr(type_, '__origin__') and type_.__origin__ is Union:
                # 处理 Union 类型
                return f"Union[{', '.join(get_type_annotation(t) for t in type_.__args__)}]"
            elif isinstance(type_, type):
                return type_.__name__
            else:
                return str(type_)
        # 构建参数列表，包括类型注解
        params_with_annotations = ', '.join(f"{args_name}: {get_type_annotation(arg_type)}" for args_name, arg_type in zip(args_names, arg_types))
        # 构建函数定义字符串
        func_def = f"def {func_name}({params_with_annotations}) -> {get_type_annotation(return_type)}:\n{body}"
        # 创建一个新的命名空间
        namespace = {}
        # 将 typing 模块添加到命名空间
        import typing
        namespace['typing'] = typing
        exec(func_def, globals(), namespace)
        
        # 从命名空间获取新定义的函数
        new_func = namespace[func_name]
        
        return new_func

==> ktoolkits-0.0.8.tar.gz: ktoolkits-0.0.8/ktoolkits/agent/function_call.py <==
import json
import sys
from ktoolkits.client.base_api import BaseApi
from ktoolkits.client.base_api import GetMixin
from typing import Union,Dict,List,Optional,Any

class FunctionCall(BaseApi):

    @classmethod
    def create_tool(cls,tool_name: str,api_key: str = None) -> Optional[Dict[str, Any]]:

        api_reponse = GetMixin.get(path="tool", api_key=api_key, params={"tool_name":tool_name})

        if api_reponse.status_code != 200:
            return None
        else:
            result = {}
            result["tool_name"] = api_reponse.output.get("tool_name")
            result["tool_args"] = cls._get_args_name(api_reponse.output.get("input_parameters"))
            result["tool_func"] = cls._create_function(tool_name,result["tool_args"])
            result["tool_desc"] = api_reponse.output.get("description")
            return result
    
    @classmethod
    def _get_args_name(cls, input_parameters: str)-> List[str]:
        argsJson = json.loads(input_parameters)
        args_list =[]
        for item in argsJson:
            args_name = item.get("name")
            args_list.append(args_name)
        return args_list
    @classmethod
    def _create_function(cls, tool_name: str, tool_args: List[str]):

        func_name = f"k_function_{tool_name}"

        args_names = tool_args

        input_param = ""
        input_param += "{"

        for item in args_names:
            #todo: two params
            input_param += f"'{item}':{item},"
        
        input_param += "}"

        args_types = [Union[str,Dict[str,str]]]

        body = f"""
        import ktoolkits as ktool
        a = ktool.Runner.call(
                tool_name='{tool_name}',
                tool_input={input_param},
        )
        tool_output = a.output
        if tool_output:
            return tool_output
        else:
            return a.message
        """
        return_type = "str"

        return cls._create_dynamic_function(func_name,args_names,args_types,body,return_type)

    @classmethod
    def _create_dynamic_function(cls,func_name,args_names,arg_types,body, return_type):
        def get_type_annotation(type_):
            if hasattr(type_, '__origin__') and type_.__origin__ is Union:
                # 处理 Union 类型
                return f"Union[{', '.join(get_type_annotation(t) for t in type_.__args__)}]"
            elif isinstance(type_, type):
                return type_.__name__
            else:
                return str(type_)
        # 构建参数列表，包括类型注解
        params_with_annotations = ', '.join(f"{args_name}: {get_type_annotation(arg_type)}" for args_name, arg_type in zip(args_names, arg_types))
        # 构建函数定义字符串
        func_def = f"def {func_name}({params_with_annotations}) -> {get_type_annotation(return_type)}:\n{body}"
        # 创建一个新的命名空间
        namespace = {}
        # 将 typing 模块添加到命名空间
        import typing
        namespace['typing'] = typing
        exec(func_def, globals(), namespace)
        
        # 从命名空间获取新定义的函数
        new_func = namespace[func_name]
        
        return new_func
